#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import argparse

from apppath import AppPath

__author__ = "Christian Heider Nielsen"
__doc__ = r"""This script will open data the directory of an app"""

from apppath.system_open_path_utilities import AppPathSubDirEnum, open_app_path


def open_arg():
    """"""
    parser = argparse.ArgumentParser(description="Apppath Open Path")
    parser.add_argument("APP_NAME", metavar="Name", type=str, help="App name to open AppPath for")
    parser.add_argument(
        "--SITE",
        "-s",
        type=bool,
        default=False,
        metavar="SITE",
        help="Open user or site dirs (default: User)",
    )
    parser.add_argument(
        "--DIR",
        "-d",
        type=str,
        default="data",
        metavar="DIR",
        help="Which AppPath directory to open (default: 'data')",
    )

    args = parser.parse_args()
    project_app_path = AppPath(args.APP_NAME)

    open_app_path(project_app_path, AppPathSubDirEnum(args.DIR), args.SITE)


if __name__ == "__main__":
    open_arg()



from collections import namedtuple

import sorcery

from benchmarks.benchmark_func import benchmark_func
from warg.data_structures.named_ordered_dictionary import NOD


def returns_benchmark():
    """"""
    a = 1
    b = 2
    c = 3

    RandomABC = namedtuple("RandomABC", ("a", "b", "c"))

    def implicit_return():
        """"""
        return a, b, c

    def list_return():
        """"""
        return [a, b, c]

    def tuple_return():
        """"""
        return (a, b, c)

    def dict_return():
        """"""
        return {"a": a, "b": b, "c": c}

    def sorcery_return():
        """"""
        return sorcery.dict_of(a, b, c)

    def nod_return():
        """"""
        return NOD(a=a, b=b, c=c)

    def inferred_return():
        """"""
        return NOD.nod_of(a, b, c)

    def namedtuple_return():
        """"""
        return RandomABC(a, b, c)

    for func in (
        implicit_return,
        list_return,
        tuple_return,
        dict_return,
        namedtuple_return,
        nod_return,
        sorcery_return,
        inferred_return,
    ):
        t, res = benchmark_func(func)
        print(f"{func.__name__}: {t} seconds, {res}")


if __name__ == "__main__":
    returns_benchmark()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 08/03/2020
           """

__all__ = ["ensure_existence", "path_rmtree", "sanitise_path"]

import os
from itertools import cycle
from pathlib import Path
from shutil import rmtree
from typing import Iterable, Union


# from warg import passes_kws_to

# @passes_kws_to(rmtree) Throws error due to import issues
def path_rmtree(path: Path, **kwargs) -> None:
    """
    asses_kws_to rmtree from shutil
    :param path:
    :type path:
    :param kwargs:
    :type kwargs:"""
    rmtree(str(path), **kwargs)


def sanitise_path(
    path: Path,
    naughty_directory_symbols: Iterable[str] = (
        " ",
        ",",
        "<",
        ">",
        '"',
        "|",
        "?",
        "*",
        # ".",
        # ':',
    ),
    replacement_symbols: str = ("_",),
    preserve_file_suffix: bool = True,
) -> Path:
    """
    Opinionated path sanitisation

    https://docs.microsoft.com/en-us/windows/win32/fileio/naming-a-file

    preserve_file_suffix: assumes any path with a suffix at the end is a file and will preserve it but still replace naughty symbols

    """

    if not path.is_dir() and preserve_file_suffix:
        file_suffix = path.suffix
    else:
        file_suffix = None

    path_str_rep = str(path)
    path_str_rep = os.path.normpath(os.path.normcase(path_str_rep))

    for n, r in zip(naughty_directory_symbols, cycle(replacement_symbols)):
        path_str_rep = path_str_rep.replace(n, r, -1)

    path_out = Path(path_str_rep)
    if file_suffix is not None:
        return path_out.with_suffix(file_suffix)
    return path_out


def ensure_existence(
    out: Union[Path, str],
    *,
    enabled: bool = True,
    declare_file: bool = False,
    overwrite_on_wrong_type: bool = False,
    force_overwrite: bool = False,
    verbose: bool = False,
    sanitisation_func: callable = sanitise_path,
) -> Path:
    """

    :param verbose:
    :type verbose:
    :param overwrite_on_wrong_type:
    :type overwrite_on_wrong_type:
    :param force_overwrite:
    :type force_overwrite:
    :param declare_file:
    :type declare_file:
    :param sanitisation_func:
    :type sanitisation_func:
    :param out:
    :type out:
    :param enabled:
    :type enabled:
    :return:
    :rtype:"""
    if enabled:
        if isinstance(out, str):
            out = Path(out)

        if sanitisation_func:
            out = sanitisation_func(out)

        if not out.parent.exists():
            if verbose:
                print("Creating parents")
            out.parent.mkdir(parents=True, exist_ok=True)

        if out.is_file() or ("." in out.name and ".d" not in out.name) or declare_file:
            if (
                out.exists()
                and (out.is_dir() or ("." not in out.name and ".d" in out.name))
                and ((declare_file and overwrite_on_wrong_type) or force_overwrite)
            ):
                if verbose:
                    print("Removing tree")
                path_rmtree(out)
            if out.is_file() and not out.exists():
                out.touch(exist_ok=True)
        else:
            if (
                out.exists()
                and out.is_file()
                and ((not declare_file and overwrite_on_wrong_type) or force_overwrite)
            ):
                if verbose:
                    print("Deleting file")
                out.unlink()  # missing_ok=True)
            if not out.exists():
                out.mkdir(parents=True, exist_ok=True)

    return out


if __name__ == "__main__":

    def main():
        """"""
        ensure_existence(Path.cwd() / "exclude", force_overwrite=True)
        ensure_existence(Path.cwd() / "exclude" / "0.log")
        ensure_existence(Path.cwd() / "exclude" / "log.d")
        ensure_existence(Path.cwd() / "exclude" / "log.d" / "log.a")

        from apppath import PROJECT_APP_PATH

        ensure_existence(PROJECT_APP_PATH.user_log / "exclude" / "log.d" / "log.a")

        def recurse_test():
            """"""
            ensure_existence(Path.cwd() / "exclude" / "spodakjioj" / "log.d" / "log.a")
            ensure_existence(Path.cwd() / "exclude" / "spodakjioj" / "log.d" / "log.csv")

        recurse_test()

    def clean_naughty_file() -> None:
        """
        :rtype: None
        """
        pa = Path.cwd() / "uhas.asudh ojas.a." / "....  a -." / "   b.ci"

        print(pa, sanitise_path(pa))

    def clean_naughty_dir() -> None:
        """
        :rtype: None
        """
        pa = Path.cwd() / "uhas.asudh ojas.a." / "....  a -." / "   bci"

        print(pa, sanitise_path(pa))

    clean_naughty_file()
    clean_naughty_dir()
    # main()



# To add a new cell, type '# %%'
# To add a new markdown cell, type '# %% [markdown]'
# %% [markdown]
# Christian Heider Nielsen
# Created on 05/04/2020

# %%
from apppath import AppPath

# %%
apppath = AppPath("AppPath")
print(apppath.user_config)
print(apppath.user_log)
print(apppath.user_data)
print(apppath.user_cache)



import time

import numpy

from benchmarks.benchmark_func import benchmark_func
from draugr.multiprocessing_utilities.pooled_queue_processor import (
    PooledQueueProcessor,
    PooledQueueTask,
)


class Zeroes(PooledQueueTask):
    def call(self, batch_size, *args, tensor_size=(9, 9, 9, 9), **kwargs):
        """"""
        batch = [(numpy.zeros(tensor_size), i) for i in range(batch_size)]
        imgs = numpy.array([i[0] for i in batch], dtype=numpy.float32)
        ground_truth = numpy.array([i[1] for i in batch], dtype=numpy.float32)
        return (imgs, ground_truth)


Lamb = lambda a, tensor_size: f"{a, tensor_size}"


def Func(a, tensor_size):
    """"""
    return f"{a, tensor_size}"


def pqp_benchmark():
    """"""
    task = Zeroes()
    # task = Lamb #Error: cant be pickled
    # task = Func
    batch_size = 16
    tensor_size = (9, 9, 9, 9, 9)
    wait_time = 0.1
    samples = 100

    df = PooledQueueProcessor(
        task,
        args=[batch_size],
        kwargs={"tensor_size": tensor_size},
        max_queue_size=samples,
    )

    def get():
        """"""
        return df.get()

    def wait_get():
        """"""
        time.sleep(wait_time)
        return df.get()

    def generate():
        """"""
        return task(batch_size, tensor_size=tensor_size)

    def wait_generate():
        """"""
        time.sleep(wait_time)
        return task(batch_size, tensor_size=tensor_size)

    for func, discount in zip(
        (get, wait_get, generate, wait_generate),
        (0, samples * wait_time, 0, samples * wait_time),
    ):
        t, res = benchmark_func(func, samples)
        print(f"{func.__name__}: {t - discount} seconds")


if __name__ == "__main__":
    pqp_benchmark()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"

import time


def benchmark_func(func, times=100000):
    """"""
    start = time.time()
    result = None
    for _ in range(times):
        result = func()
    end = time.time()
    return end - start, result



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Neo documentation build configuration file, created by
# sphinx-quickstart on Tue Jul 25 10:23:12 2017.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#

import sys
from pathlib import Path

PACKAGE_ROOT = Path(__file__).parent.parent
sys.path.insert(0, PACKAGE_ROOT.absolute())

from apppath import PROJECT_AUTHOR, PROJECT_NAME, PROJECT_YEAR, PROJECT_VERSION

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.

extensions = [
    "m2r2",
    # 'recommonmark',
    "sphinxcontrib.programoutput",
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.napoleon",
    "sphinx.ext.doctest",
    "sphinx.ext.intersphinx",
    "sphinx.ext.todo",
    "sphinx.ext.coverage",
    "sphinx.ext.mathjax",
    "sphinx.ext.viewcode",
    "sphinx.ext.githubpages",
    "sphinx.ext.graphviz",
]

napoleon_use_ivar = True

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
# source_suffix = '.rst'
source_suffix = {".rst": "restructuredtext", ".txt": "markdown", ".md": "markdown"}

# source_parsers = {
#    '.md': CommonMarkParser,
# }

# The master toctree document.
master_doc = "index"

# General information about the project.
project = PROJECT_NAME
author = PROJECT_AUTHOR
copyright_text = f"{PROJECT_YEAR}, {PROJECT_AUTHOR}"

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
version = PROJECT_VERSION
release = version  # 'master'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set 'language' from the command line for these cases.
language = None

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = []

# The name of the Pygments (syntax highlighting) style to use.
highlight_language = "python"
pygments_style = "sphinx"

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = True

# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "alabaster"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named 'default.css' will overwrite the builtin 'default.css'.
html_static_path = ["_static"]

html_baseurl = f"{PROJECT_NAME}.github.io"

# -- Options for HTMLHelp output ------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = f"{PROJECT_NAME}doc"

# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',
    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',
    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',
    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (
        master_doc,
        f"{PROJECT_NAME}.tex",
        f"{PROJECT_NAME} Documentation",
        PROJECT_AUTHOR,
        "manual",
    )
]

# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [(master_doc, PROJECT_NAME, f"{PROJECT_NAME} Documentation", [PROJECT_AUTHOR], 1)]

# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        master_doc,
        PROJECT_NAME,
        f"{PROJECT_NAME} Documentation",
        PROJECT_AUTHOR,
        PROJECT_NAME,
        "One line description of project.",
        "Miscellaneous",
    )
]

# -- Options for Epub output ----------------------------------------------

# Bibliographic Dublin Core info.
epub_title = PROJECT_NAME
epub_author = PROJECT_AUTHOR
epub_publisher = PROJECT_AUTHOR
epub_copyright = copyright_text

# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
#
# epub_identifier = ''

# A unique identification for the text.
#
# epub_uid = ''

# A list of files that should not be packed into the epub file.
epub_exclude_files = ["search.html"]

# Example configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {
    "python": ("https://docs.python.org/", None),
    "numpy": ("http://docs.scipy.org/doc/numpy/", None),
}

# -- A patch that prevents Sphinx from cross-referencing ivar tags -------
# See http://stackoverflow.com/a/41184353/3343043

from docutils import nodes
from sphinx import addnodes
from sphinx.util.docfields import TypedField


def patched_make_field(self, types, domain, items, **kw):
    """"""

    # `kw` catches `env=None` needed for newer sphinx while maintaining
    #  backwards compatibility when passed along further down!
    # #type: (List, unicode, Tuple) -> nodes.field
    def handle_item(fieldarg, content):
        """"""
        par = nodes.paragraph()
        par += addnodes.literal_strong("", fieldarg)  # Patch: this line added
        # par.extend(self.make_xrefs(self.rolename, domain, fieldarg,
        #                           addnodes.literal_strong))
        if fieldarg in types:
            par += nodes.Text(" (")
            # NOTE: using .pop() here to prevent a single type node to be
            # inserted twice into the doctree, which leads to
            # inconsistencies later when references are resolved
            fieldtype = types.pop(fieldarg)
            if len(fieldtype) == 1 and isinstance(fieldtype[0], nodes.Text):
                typename = "".join(n.astext() for n in fieldtype)
                typename = typename.replace("int", "python:int")
                typename = typename.replace("long", "python:long")
                typename = typename.replace("float", "python:float")
                typename = typename.replace("type", "python:type")
                par.extend(
                    self.make_xrefs(
                        self.typerolename,
                        domain,
                        typename,
                        addnodes.literal_emphasis,
                        **kw,
                    )
                )
            else:
                par += fieldtype
            par += nodes.Text(")")
        par += nodes.Text(" -- ")
        par += content
        return par

    field_name = nodes.field_name("", self.label)
    if len(items) == 1 and self.can_collapse:
        field_arg, content = items[0]
        body_node = handle_item(field_arg, content)
    else:
        body_node = self.list_type()
        for field_arg, content in items:
            body_node += nodes.list_item("", handle_item(field_arg, content))
    field_body = nodes.field_body("", body_node)
    return nodes.field("", field_name, field_body)


TypedField.make_field = patched_make_field



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 05/04/2020
           """

from apppath import AppPath

if __name__ == "__main__":

    def main():
        """"""
        apppath = AppPath("AppPath")
        print(apppath.user_config)
        print(apppath.user_log)
        print(apppath.user_data)
        print(apppath.user_cache)

    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import argparse
from shutil import rmtree

from apppath import AppPath

__author__ = "Christian Heider Nielsen"
__doc__ = r"""This script will clean a apppath directory of an app"""


def clean_arg():
    """"""
    parser = argparse.ArgumentParser(description="Apppath Clean Path")
    parser.add_argument("APP_NAME", metavar="Name", type=str, help="App name to clean AppPath for")
    parser.add_argument(
        "--DIR",
        "-d",
        type=str,
        default="cache",
        metavar="DIR",
        help="Which AppPath directory to clean (default: 'cache')",
    )

    args = parser.parse_args()
    project_app_path = AppPath(args.APP_NAME)

    if args.DIR == "data":
        directory = project_app_path.user_data
    elif args.DIR == "config":
        directory = project_app_path.user_config
    elif args.DIR == "cache":
        directory = project_app_path.user_cache
    elif args.DIR == "logs":
        directory = project_app_path.user_log
    else:
        raise NotADirectoryError(args.DIR)

    print(f"Wiping {directory}")
    if directory.exists():
        rmtree(directory)
    else:
        directory.mkdir()


if __name__ == "__main__":
    clean_arg()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys

__author__ = "Christian Heider Nielsen"
__doc__ = ""

__all__ = ["get_win_folder", "SYSTEM"]

from typing import Any

PY3 = sys.version_info[0] == 3

if PY3:
    unicode = str


def _get_win_folder_from_registry(csidl_name: Any) -> Any:
    """This is a fallback technique at best. I'm not sure if using the
    registry for this guarantees us the correct answer for all CSIDL_*
    names."""
    if PY3:
        import winreg as _winreg
    else:
        import _winreg

    shell_folder_name = {
        "CSIDL_APPDATA": "AppData",
        "CSIDL_COMMON_APPDATA": "Common AppData",
        "CSIDL_LOCAL_APPDATA": "Local AppData",
    }[csidl_name]

    key = _winreg.OpenKey(
        _winreg.HKEY_CURRENT_USER,
        r"Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders",
    )
    dir, type = _winreg.QueryValueEx(key, shell_folder_name)
    return dir


def _get_win_folder_with_pywin32(csidl_name: Any) -> Any:
    from win32com.shell import shellcon, shell

    dir = shell.SHGetFolderPath(0, getattr(shellcon, csidl_name), 0, 0)
    # Try to make this a unicode path because SHGetFolderPath does
    # not return unicode strings when there is unicode data in the
    # path.
    try:
        dir = unicode(dir)

        # Downgrade to short path name if have highbit chars. See
        # <http://bugs.activestate.com/show_bug.cgi?id=85099>.
        has_high_char = False
        for c in dir:
            if ord(c) > 255:
                has_high_char = True
                break
        if has_high_char:
            try:
                import win32api

                dir = win32api.GetShortPathName(dir)
            except ImportError:
                pass
    except UnicodeError:
        pass
    return dir


def _get_win_folder_with_ctypes(csidl_name: Any) -> Any:
    from ctypes import windll, create_unicode_buffer

    csidl_const = {
        "CSIDL_APPDATA": 26,
        "CSIDL_COMMON_APPDATA": 35,
        "CSIDL_LOCAL_APPDATA": 28,
    }[csidl_name]

    buf = create_unicode_buffer(1024)
    windll.shell32.SHGetFolderPathW(None, csidl_const, None, 0, buf)

    # Downgrade to short path name if have highbit chars. See
    # <http://bugs.activestate.com/show_bug.cgi?id=85099>.
    has_high_char = False
    for c in buf:
        if ord(c) > 255:
            has_high_char = True
            break
    if has_high_char:
        buf2 = create_unicode_buffer(1024)
        if windll.kernel32.GetShortPathNameW(buf.value, buf2, 1024):
            buf = buf2

    return buf.value


def _get_win_folder_with_jna(csidl_name: Any) -> Any:
    import array
    from com.sun import jna
    from com.sun.jna.platform import win32

    buf_size = win32.WinDef.MAX_PATH * 2
    buf = array.zeros("c", buf_size)
    shell = win32.Shell32.INSTANCE
    shell.SHGetFolderPath(
        None,
        getattr(win32.ShlObj, csidl_name),
        None,
        win32.ShlObj.SHGFP_TYPE_CURRENT,
        buf,
    )
    dir = jna.Native.toString(buf.tostring()).rstrip("\0")

    # Downgrade to short path name if have highbit chars. See
    # <http://bugs.activestate.com/show_bug.cgi?id=85099>.
    has_high_char = False
    for c in dir:
        if ord(c) > 255:
            has_high_char = True
            break
    if has_high_char:
        buf = array.zeros("c", buf_size)
        kernel = win32.Kernel32.INSTANCE
        if kernel.GetShortPathName(dir, buf, buf_size):
            dir = jna.Native.toString(buf.tostring()).rstrip("\0")

    return dir


get_win_folder = None

if sys.platform.startswith("java"):
    import platform

    os_name = platform.java_ver()[3][0]
    if os_name.startswith("Windows"):  # "Windows XP", "Windows 7", etc.
        SYSTEM = "win32"
    elif os_name.startswith("Mac"):  # "Mac OS X", etc.
        SYSTEM = "darwin"
    else:  # "Linux", "SunOS", "FreeBSD", etc.
        # Setting this to "linux2" is not ideal, but only Windows or Mac
        # are actually checked for and the rest of the module expects
        # *sys.platform* style strings.
        SYSTEM = "linux2"
else:
    SYSTEM = sys.platform

if SYSTEM == "win32":  # IMPORT TESTS
    try:
        from win32com import shell

        get_win_folder = _get_win_folder_with_pywin32
    except ImportError:
        try:
            from ctypes import windll

            get_win_folder = _get_win_folder_with_ctypes
        except ImportError:
            try:
                from com.sun import jna

                get_win_folder = _get_win_folder_with_jna
            except ImportError:
                get_win_folder = _get_win_folder_from_registry



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import shutil
from pathlib import Path

from apppath.utilities import SYSTEM, ensure_existence, get_win_folder

__author__ = "Christian Heider Nielsen"
__doc__ = "Application data directories extension for pathlib"

__all__ = ["AppPath"]


class AppPath(object):
    r"""
    AppPath class for easing cross platform access to proper app data directories
    This class is an abstraction for getting system conventional application paths for data, logs, etc."""

    def __init__(
        self,
        app_name: str,
        app_author: str = None,
        app_version: str = None,
        roaming: bool = False,
        multi_path: bool = False,
        ensure_existence_on_access: bool = True,
        normalise_path: bool = True,
    ):
        r"""
        Typical user data directories are:
        Mac OS X:               ~/Library/Application Support/<AppName>
        Unix:                   ~/.local/share/<AppName>    # or in $XDG_DATA_HOME, if defined
        Win XP (not roaming):   C:\Documents and Settings\<username>\Application Data\<AppAuthor>\<AppName>
        Win XP (roaming):       C:\Documents and Settings\<username>\Local Settings\Application
        Data\<AppAuthor>\<AppName>
        Win 7  (not roaming):   C:\Users\<username>\AppData\Local\<AppAuthor>\<AppName>
        Win 7  (roaming):       C:\Users\<username>\AppData\Roaming\<AppAuthor>\<AppName>

        Typical site data directories are:
        Mac OS X:   /Library/Application Support/<AppName>
        Unix:       /usr/local/share/<AppName> or /usr/share/<AppName>
        Win XP:     C:\Documents and Settings\All Users\Application Data\<AppAuthor>\<AppName>
        Vista:      (Fail! "C:\ProgramData" is a hidden *system* directory on Vista.)
        Win 7:      C:\ProgramData\<AppAuthor>\<AppName>   # Hidden, but writeable on Win 7.

        Typical user config directories are:
        Mac OS X:               ~/Library/Preferences/<AppName>
        Unix:                   ~/.config/<AppName>     # or in $XDG_CONFIG_HOME, if defined
        Win *:                  same as user_data_dir

        Typical site config directories are:
        Mac OS X:   same as site_data_dir
        Unix:       /etc/xdg/<AppName> or $XDG_CONFIG_DIRS[i]/<AppName> for each value in
        $XDG_CONFIG_DIRS
        Win *:      same as site_data_dir
        Vista:      (Fail! "C:\ProgramData" is a hidden *system* directory on Vista.)

        Typical user log directories are:
        Mac OS X:   ~/Library/Logs/<AppName>
        Unix:       ~/.cache/<AppName>/log  # or under $XDG_CACHE_HOME if defined
        Win XP:     C:\Documents and Settings\<username>\Local Settings\Application
        Data\<AppAuthor>\<AppName>\Logs
        Vista:      C:\Users\<username>\AppData\Local\<AppAuthor>\<AppName>\Logs

        Typical user state directories are:
        Mac OS X:  same as user_data_dir
        Unix:      ~/.local/state/<AppName>   # or in $XDG_STATE_HOME, if defined
        Win *:     same as user_data_dir

        Typical user cache directories are:
        Mac OS X:   ~/Library/Caches/<AppName>
        Unix:       ~/.cache/<AppName> (XDG default)
        Win XP:     C:\Documents and Settings\<username>\Local Settings\Application
        Data\<AppAuthor>\<AppName>\Cache
        Vista:      C:\Users\<username>\AppData\Local\<AppAuthor>\<AppName>\Cache

        :param app_name:
        :param app_author:
        :param app_version:
        :param roaming: "roaming" (boolean, default False) can be set True to use the Windows roaming appdata directory. That means that for users on a Windows network setup for roaming profiles, this user data will be sync'd on login. See <http://technet.microsoft.com/en-us/library/cc766489(WS.10).aspx> for a discussion of issues.
        :param multi_path: "multi_path" is an optional parameter only applicable to *nix which indicates that the entire list of data dirs should be returned. By default, the first item from XDG_DATA_DIRS is returned, or '/usr/local/share/<AppName>', if XDG_DATA_DIRS is not set
        :param ensure_existence_on_access:
        :param normalise_path:"""
        assert isinstance(app_name, str)
        if normalise_path:
            app_name = app_name.strip().lower().replace(" ", "_")
        self._app_name = app_name
        if isinstance(app_author, str):
            if normalise_path:
                app_author = app_author.strip().lower().replace(" ", "_")
        else:
            assert app_author is None
        self._app_author = app_author
        if isinstance(app_version, (int, float)):
            app_version = str(app_version)
        if isinstance(app_version, str):
            if normalise_path:
                app_version = app_version.strip().lower().replace(" ", "_")
        self._app_version = app_version
        self._roaming = roaming
        self._multi_path = multi_path
        self._ensure_existence = ensure_existence_on_access

    def __divmod__(self, other):
        return self.__truediv__(other)

    def __truediv__(self, other):
        raise ArithmeticError(
            "The AppPath class itself is not a Path, you should use one of it path properties ("
            'e.g. ".user_data"  or ".user_config")'
        )

    @property
    def user_data(self) -> Path:
        """
        User data path

        :return:
        :rtype:"""

        return ensure_existence(
            self._user_data_path(
                self._app_name,
                self._app_author,
                version=self._app_version,
                roaming=self._roaming,
            ),
            enabled=self._ensure_existence,
        )

    @property
    def site_data(self) -> Path:
        """
        Site data path

        :return:
        :rtype:"""
        return ensure_existence(
            self._site_data_path(
                self._app_name,
                self._app_author,
                version=self._app_version,
                multi_path=self._multi_path,
            ),
            enabled=self._ensure_existence,
        )

    @property
    def user_config(self) -> Path:
        """
        User config path

        :return:
        :rtype:"""
        return ensure_existence(
            self._user_config_path(
                self._app_name,
                self._app_author,
                version=self._app_version,
                roaming=self._roaming,
            ),
            enabled=self._ensure_existence,
        )

    @property
    def site_config(self) -> Path:
        """
        Site config path

        :return:
        :rtype:"""
        return ensure_existence(
            self._site_config_path(
                self._app_name,
                self._app_author,
                version=self._app_version,
                multi_path=self._multi_path,
            ),
            enabled=self._ensure_existence,
        )

    @property
    def root_cache(self) -> Path:
        """

        :return:
        """
        if SYSTEM == "win32" or SYSTEM == "win32":
            raise SystemError("Invalid system")
        return ensure_existence(Path("/var/cache") / self._app_name, enabled=self._ensure_existence)

    @property
    def root_config(self) -> Path:
        """

        :return:
        """
        if SYSTEM == "win32" or SYSTEM == "win32":
            raise SystemError("Invalid system")
        return ensure_existence(Path("/etc") / self._app_name, enabled=self._ensure_existence)

    @property
    def root_log(self) -> Path:
        """

        :return:
        """
        if SYSTEM == "win32" or SYSTEM == "win32":
            raise SystemError("Invalid system")
        return ensure_existence(Path("/var/log") / self._app_name, enabled=self._ensure_existence)

    @property
    def root_state(self) -> Path:
        """

        :return:
        """
        if SYSTEM == "win32" or SYSTEM == "win32":
            raise SystemError("Invalid system")
        return ensure_existence(Path("/var/lib") / self._app_name, enabled=self._ensure_existence)

    @property
    def root_run(self) -> Path:
        """

        :return:
        """
        if SYSTEM == "win32" or SYSTEM == "win32":
            raise SystemError("Invalid system")
        return ensure_existence(Path("/run") / self._app_name, enabled=self._ensure_existence)

    @property
    def root_tmp(self) -> Path:
        """

        :return:
        """
        if SYSTEM == "win32" or SYSTEM == "win32":
            raise SystemError("Invalid system")
        return ensure_existence(Path("/tmp") / self._app_name, enabled=self._ensure_existence)

    @property
    def root_long_tmp(self) -> Path:
        """

        :return:
        """
        if SYSTEM == "win32" or SYSTEM == "win32":
            raise SystemError("Invalid system")
        return ensure_existence(Path("/var/tmp") / self._app_name, enabled=self._ensure_existence)

    @property
    def site_cache(self) -> Path:
        """
        TODO: Nonsense, is same as user_cache
        Returns:

        """
        return self.user_cache

    @property
    def site_log(self) -> Path:
        """
        TODO: Nonsense, is same as user_log
        Returns:

        """
        return self.user_log

    @property
    def user_cache(self) -> Path:
        """
        User cache path

        :return:
        :rtype:"""
        return ensure_existence(
            self._user_cache_path(self._app_name, self._app_author, version=self._app_version),
            enabled=self._ensure_existence,
        )

    @property
    def user_state(self) -> Path:
        """
        User state path

        :return:
        :rtype:"""
        return ensure_existence(
            self._user_state_path(self._app_name, self._app_author, version=self._app_version),
            enabled=self._ensure_existence,
        )

    @property
    def user_log(self) -> Path:
        """
        User log path

        :return:
        :rtype:"""
        return ensure_existence(
            self._user_log_path(self._app_name, self._app_author, version=self._app_version),
            enabled=self._ensure_existence,
        )

    @staticmethod
    def _user_data_path(
        app_name: str = None,
        app_author: str = None,
        version: str = None,
        roaming: bool = False,
    ) -> Path:
        r"""Return full path to the user-specific data dir for this application.

        "app_name" is the name of application.
        If None, just the system directory is returned.
        "app_author" (only used on Windows) is the name of the
        app_author or distributing body for this application. Typically
        it is the owning company name. This falls back to app_name. You may
        pass False to disable it.
        "version" is an optional version path element to append to the
        path. You might want to use this if you want multiple versions
        of your app to be able to run independently. If used, this
        would typically be "<major>.<minor>".
        Only applied when app_name is present.
        "roaming" (boolean, default False) can be set True to use the Windows
        roaming appdata directory. That means that for users on a Windows
        network setup for roaming profiles, this user data will be
        sync'd on login. See
        <http://technet.microsoft.com/en-us/library/cc766489(WS.10).aspx>
        for a discussion of issues.


        Notes:
        - MSDN on where to store app data files:
        http://support.microsoft.com/default.aspx?scid=kb;en-us;310294#XSLTH3194121123120121120120
        - Mac OS X: http://developer.apple.com/documentation/MacOSX/Conceptual/BPFileSystem/index.html
        - XDG spec for Un*x: https://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html

        Typical user data directories are:
        Mac OS X:               ~/Library/Application Support/<AppName>
        Unix:                   ~/.local/share/<AppName>    # or in $XDG_DATA_HOME, if defined
        Win XP (not roaming):   C:\Documents and Settings\<username>\Application Data\<AppAuthor>\<AppName>
        Win XP (roaming):       C:\Documents and Settings\<username>\Local Settings\Application
        Data\<AppAuthor>\<AppName>
        Win 7  (not roaming):   C:\Users\<username>\AppData\Local\<AppAuthor>\<AppName>
        Win 7  (roaming):       C:\Users\<username>\AppData\Roaming\<AppAuthor>\<AppName>

        For Unix, we follow the XDG spec and support $XDG_DATA_HOME.
        That means, by default "~/.local/share/<AppName>"."""

        if SYSTEM == "win32":
            if app_author is None:
                app_author = app_name
            const = roaming and "CSIDL_APPDATA" or "CSIDL_LOCAL_APPDATA"
            path_ = Path(os.path.normpath(get_win_folder(const)))
            if app_name:
                if app_author is not False:
                    path_ = path_ / app_author / app_name
                else:
                    path_ /= app_name
        elif SYSTEM == "darwin":
            path_ = Path.home() / "Library" / "Application Support"
            if app_name:
                path_ /= app_name
        else:
            path_ = Path(os.getenv("XDG_DATA_HOME", Path.home() / ".local" / "share"))
            if app_name:
                path_ /= app_name
        if app_name and version:
            path_ /= version
        return path_

    @staticmethod
    def _site_data_path(
        app_name: str = None,
        app_author: str = None,
        version: str = None,
        multi_path: bool = False,
    ) -> Path:
        r"""Return full path to the user-shared data dir for this application.

        "app_name" is the name of application.
        If None, just the system directory is returned.
        "app_author" (only used on Windows) is the name of the
        app_author or distributing body for this application. Typically
        it is the owning company name. This falls back to app_name. You may
        pass False to disable it.
        "version" is an optional version path element to append to the
        path. You might want to use this if you want multiple versions
        of your app to be able to run independently. If used, this
        would typically be "<major>.<minor>".
        Only applied when app_name is present.
        "multi_path" is an optional parameter only applicable to *nix
        which indicates that the entire list of data dirs should be
        returned. By default, the first item from XDG_DATA_DIRS is
        returned, or '/usr/local/share/<AppName>',
        if XDG_DATA_DIRS is not set

        Typical site data directories are:
        Mac OS X:   /Library/Application Support/<AppName>
        Unix:       /usr/local/share/<AppName> or /usr/share/<AppName>
        Win XP:     C:\Documents and Settings\All Users\Application Data\<AppAuthor>\<AppName>
        Vista:      (Fail! "C:\ProgramData" is a hidden *system* directory on Vista.)
        Win 7:      C:\ProgramData\<AppAuthor>\<AppName>   # Hidden, but writeable on Win 7.

        For Unix, this is using the $XDG_DATA_DIRS[0] default.

        WARNING: Do not use this on Windows. See the Vista-Fail note above for why."""
        if SYSTEM == "win32":
            if app_author is None:
                app_author = app_name
            path = Path(os.path.normpath(get_win_folder("CSIDL_COMMON_APPDATA")))
            if app_name:
                if app_author is not False:
                    path = path / app_author / app_name
                else:
                    path /= app_name
        elif SYSTEM == "darwin":
            path = Path.home() / "Library" / "Application Support"
            if app_name:
                path /= app_name
        else:
            # XDG default for $XDG_DATA_DIRS
            # only first, if multipath is False
            path = os.getenv("XDG_DATA_DIRS", os.pathsep.join(["/usr/local/share", "/usr/share"]))
            path_list = [os.path.expanduser(x.rstrip(os.sep)) for x in path.split(os.pathsep)]
            if app_name:
                if version:
                    app_name = Path(app_name) / version
                path_list = [Path(x) / app_name for x in path_list]

            if multi_path:
                path = os.pathsep.join([str(a) for a in path_list])
            else:
                path_list = [Path(a) for a in path_list]
                path = path_list[0]
            return path

        if app_name and version:
            path /= version
        return path

    @staticmethod
    def _user_config_path(
        app_name: str = None,
        app_author: str = None,
        version: str = None,
        roaming: bool = False,
    ) -> Path:
        r"""Return full path to the user-specific config dir for this application.

        "app_name" is the name of application.
        If None, just the system directory is returned.
        "app_author" (only used on Windows) is the name of the
        app_author or distributing body for this application. Typically
        it is the owning company name. This falls back to app_name. You may
        pass False to disable it.
        "version" is an optional version path element to append to the
        path. You might want to use this if you want multiple versions
        of your app to be able to run independently. If used, this
        would typically be "<major>.<minor>".
        Only applied when app_name is present.
        "roaming" (boolean, default False) can be set True to use the Windows
        roaming appdata directory. That means that for users on a Windows
        network setup for roaming profiles, this user data will be
        sync'd on login. See
        <http://technet.microsoft.com/en-us/library/cc766489(WS.10).aspx>
        for a discussion of issues.

        Typical user config directories are:
        Mac OS X:               ~/Library/Preferences/<AppName>
        Unix:                   ~/.config/<AppName>     # or in $XDG_CONFIG_HOME, if defined
        Win *:                  same as user_data_dir

        For Unix, we follow the XDG spec and support $XDG_CONFIG_HOME.
        That means, by default "~/.config/<AppName>"."""
        if SYSTEM == "win32":
            path = AppPath._user_data_path(app_name, app_author, None, roaming)
        elif SYSTEM == "darwin":
            path = Path.home() / "Library" / "Preferences"
            if app_name:
                path /= app_name
        else:
            path = Path(os.getenv("XDG_CONFIG_HOME", Path.home() / ".config"))
            if app_name:
                path /= app_name
        if app_name and version:
            path /= version
        return path

    @staticmethod
    def _site_config_path(
        app_name: str = None,
        app_author: str = None,
        version: str = None,
        multi_path: bool = False,
    ) -> Path:
        r"""Return full path to the user-shared data dir for this application.

        "app_name" is the name of application.
        If None, just the system directory is returned.
        "app_author" (only used on Windows) is the name of the
        app_author or distributing body for this application. Typically
        it is the owning company name. This falls back to appname. You may
        pass False to disable it.
        "version" is an optional version path element to append to the
        path. You might want to use this if you want multiple versions
        of your app to be able to run independently. If used, this
        would typically be "<major>.<minor>".
        Only applied when app_name is present.
        "multi_path" is an optional parameter only applicable to *nix
        which indicates that the entire list of config dirs should be
        returned. By default, the first item from XDG_CONFIG_DIRS is
        returned, or '/etc/xdg/<AppName>', if XDG_CONFIG_DIRS is not set

        Typical site config directories are:
        Mac OS X:   same as site_data_dir
        Unix:       /etc/xdg/<AppName> or $XDG_CONFIG_DIRS[i]/<AppName> for each value in
        $XDG_CONFIG_DIRS
        Win *:      same as site_data_dir
        Vista:      (Fail! "C:\ProgramData" is a hidden *system* directory on Vista.)

        For Unix, this is using the $XDG_CONFIG_DIRS[0] default, if multipath=False

        WARNING: Do not use this on Windows. See the Vista-Fail note above for why."""
        if SYSTEM == "win32":
            path = AppPath._site_data_path(app_name, app_author)
            if app_name and version:
                path /= version
        elif SYSTEM == "darwin":
            path = Path.home() / "Library" / "Preferences"
            if app_name:
                path /= app_name
        else:
            # XDG default for $XDG_CONFIG_DIRS
            # only first, if multi_path is False
            path = os.getenv("XDG_CONFIG_DIRS", "/etc/xdg")
            path_list = [os.path.expanduser(x.rstrip(os.sep)) for x in path.split(os.pathsep)]
            if app_name:
                if version:
                    app_name = Path(app_name) / version
                path_list = [Path(x) / app_name for x in path_list]

            if multi_path:
                path = os.pathsep.join([str(a) for a in path_list])
            else:
                # path_list = [Path(a) for a in path_list]
                path = path_list[0]
        return path

    @staticmethod
    def _user_cache_path(
        app_name: str = None,
        app_author: str = None,
        version: str = None,
        opinionated: bool = True,
    ) -> Path:
        r"""Return full path to the user-specific cache dir for this application.

        "appname" is the name of application.
        If None, just the system directory is returned.
        "appauthor" (only used on Windows) is the name of the
        appauthor or distributing body for this application. Typically
        it is the owning company name. This falls back to appname. You may
        pass False to disable it.
        "version" is an optional version path element to append to the
        path. You might want to use this if you want multiple versions
        of your app to be able to run independently. If used, this
        would typically be "<major>.<minor>".
        Only applied when appname is present.
        "opinionated" (boolean) can be False to disable the appending of
        "Cache" to the base app data dir for Windows. See
        discussion below.

        Typical user cache directories are:
        Mac OS X:   ~/Library/Caches/<AppName>
        Unix:       ~/.cache/<AppName> (XDG default)
        Win XP:     C:\Documents and Settings\<username>\Local Settings\Application
        Data\<AppAuthor>\<AppName>\Cache
        Vista:      C:\Users\<username>\AppData\Local\<AppAuthor>\<AppName>\Cache

        On Windows the only suggestion in the MSDN docs is that local settings go in
        the `CSIDL_LOCAL_APPDATA` directory. This is identical to the non-roaming
        app data dir (the default returned by `user_data_dir` above). Apps typically
        put cache data somewhere *under* the given dir here. Some examples:
        ...\Mozilla\Firefox\Profiles\<ProfileName>\Cache
        ...\Acme\SuperApp\Cache\1.0
        OPINION: This function appends "Cache" to the `CSIDL_LOCAL_APPDATA` value.
        This can be disabled with the `opinionated=False` option."""
        if SYSTEM == "win32":
            if app_author is None:
                app_author = app_name
            path = Path(os.path.normpath(get_win_folder("CSIDL_LOCAL_APPDATA")))
            if app_name:
                if app_author is not False:
                    path = path / app_author / app_name
                else:
                    path /= app_name
                if opinionated:
                    path /= "Cache"
        elif SYSTEM == "darwin":
            path = Path.home() / "Library" / "Caches"
            if app_name:
                path /= app_name
        else:
            path = Path(os.getenv("XDG_CACHE_HOME", Path.home() / ".cache"))
            if app_name:
                path /= app_name
        if app_name and version:
            path /= version
        return path

    @staticmethod
    def _user_state_path(
        app_name: str = None,
        app_author: str = None,
        version: str = None,
        roaming: bool = False,
    ) -> Path:
        r"""Return full path to the user-specific state dir for this application.

        "app_name" is the name of application.
        If None, just the system directory is returned.
        "app_author" (only used on Windows) is the name of the
        app_author or distributing body for this application. Typically
        it is the owning company name. This falls back to appname. You may
        pass False to disable it.
        "version" is an optional version path element to append to the
        path. You might want to use this if you want multiple versions
        of your app to be able to run independently. If used, this
        would typically be "<major>.<minor>".
        Only applied when app_name is present.
        "roaming" (boolean, default False) can be set True to use the Windows
        roaming appdata directory. That means that for users on a Windows
        network setup for roaming profiles, this user data will be
        sync'd on login. See
        <http://technet.microsoft.com/en-us/library/cc766489(WS.10).aspx>
        for a discussion of issues.

        Typical user state directories are:
        Mac OS X:  same as user_data_dir
        Unix:      ~/.local/state/<AppName>   # or in $XDG_STATE_HOME, if defined
        Win *:     same as user_data_dir

        For Unix, we follow this Debian proposal <https://wiki.debian.org/XDGBaseDirectorySpecification#state>
        to extend the XDG spec and support $XDG_STATE_HOME.

        That means, by default "~/.local/state/<AppName>"."""
        if SYSTEM in ["win32", "darwin"]:
            path = AppPath._user_data_path(app_name, app_author, None, roaming)
        else:
            path = Path(os.getenv("XDG_STATE_HOME", Path.home() / ".local" / "state"))
            if app_name:
                path /= app_name
        if app_name and version:
            path /= version
        return path

    @staticmethod
    def _user_log_path(
        app_name: str = None,
        app_author: str = None,
        version: str = None,
        opinionated: bool = True,
    ) -> Path:
        r"""Return full path to the user-specific log dir for this application.

        "app_name" is the name of application.
        If None, just the system directory is returned.
        "app_author" (only used on Windows) is the name of the
        app_author or distributing body for this application. Typically
        it is the owning company name. This falls back to appname. You may
        pass False to disable it.
        "version" is an optional version path element to append to the
        path. You might want to use this if you want multiple versions
        of your app to be able to run independently. If used, this
        would typically be "<major>.<minor>".
        Only applied when app_name is present.
        "opinionated" (boolean) can be False to disable the appending of
        "Logs" to the base app data dir for Windows, and "log" to the
        base cache dir for Unix. See discussion below.

        Typical user log directories are:
        Mac OS X:   ~/Library/Logs/<AppName>
        Unix:       ~/.cache/<AppName>/log  # or under $XDG_CACHE_HOME if defined
        Win XP:     C:\Documents and Settings\<username>\Local Settings\Application
        Data\<AppAuthor>\<AppName>\Logs
        Vista:      C:\Users\<username>\AppData\Local\<AppAuthor>\<AppName>\Logs

        On Windows the only suggestion in the MSDN docs is that local settings
        go in the `CSIDL_LOCAL_APPDATA` directory. (Note: I'm interested in
        examples of what some windows apps use for a logs dir.)

        OPINION: This function appends "Logs" to the `CSIDL_LOCAL_APPDATA`
        value for Windows and appends "log" to the user cache dir for Unix.
        This can be disabled with the `opinionated=False` option."""
        if SYSTEM == "darwin":
            path = Path.home() / "Library" / "Logs" / app_name
        elif SYSTEM == "win32":
            path = AppPath._user_data_path(app_name, app_author, version)
            version = False
            if opinionated:
                path /= "Logs"
        else:
            path = AppPath._user_cache_path(app_name, app_author, version)
            version = False
            if opinionated:
                path /= "log"
        if app_name and version:
            path /= version
        return path

    def clean(self, confirm: bool = True) -> None:
        """

            Remove all apppath data

        :param confirm:
        :type confirm:"""
        if confirm:
            self.clean_user_log()
            self.clean_user_config()
            self.clean_user_data()
            self.clean_user_cache()
            self.clean_site_config()
            self.clean_site_data()

    def clean_user_log(self) -> None:
        """"""
        if self.user_log.exists():
            shutil.rmtree(self.user_log)

    def clean_user_config(self) -> None:
        """"""
        if self.user_config.exists():
            shutil.rmtree(self.user_config)

    def clean_user_data(self) -> None:
        """"""
        if self.user_data.exists():
            shutil.rmtree(self.user_data)

    def clean_user_cache(self) -> None:
        """"""
        if self.user_cache.exists():
            shutil.rmtree(self.user_cache)

    def clean_site_config(self) -> None:
        """"""
        if self.site_config.exists():
            shutil.rmtree(self.site_config)

    def clean_site_data(self) -> None:
        """"""
        if self.site_data.exists():
            shutil.rmtree(self.site_data)

    @property
    def app_name(self):
        """"""
        return self._app_name

    @property
    def app_author(self):
        """"""
        return self._app_author

    def app_version(self):
        """"""
        return self._app_version


if __name__ == "__main__":

    def main():
        """"""
        _app_name = "MyApp"
        _app_author = __author__

        props = (
            "user_data",
            "user_config",
            "user_cache",
            "user_state",
            "user_log",
            "site_data",
            "site_config",
        )

        print("-- app dirs (with optional 'version')")
        dirs = AppPath(_app_name, _app_author, app_version="1.0", ensure_existence_on_access=False)
        for prop in props:
            print(f"{prop}: {getattr(dirs, prop)}")

        print("\n-- app dirs (without optional 'version')")
        dirs = AppPath(_app_name, _app_author, ensure_existence_on_access=False)
        for prop in props:
            print(f"{prop}: {getattr(dirs, prop)}")

        print("\n-- app dirs (without optional '_app_author')")
        dirs = AppPath(_app_name, ensure_existence_on_access=False)
        for prop in props:
            print(f"{prop}: {getattr(dirs, prop)}")

        print("\n-- app dirs (with disabled '_app_author')")
        dirs = AppPath(_app_name, ensure_existence_on_access=False)
        for prop in props:
            print(f"{prop}: {getattr(dirs, prop)}")

        ex = None
        try:
            dirs / "a"
        except Exception as e:
            ex = e
        assert ex is not None, "expected exception"

    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 31-01-2021
           """

import subprocess
import sys
from enum import Enum
from pathlib import Path

from apppath.app_path import AppPath

__all__ = ["AppPathSubDirEnum", "open_app_path", "system_open_path"]


class AppPathSubDirEnum(Enum):
    """ """

    data = "data"
    config = "config"
    cache = "cache"
    log = "log"


def system_open_path(path: Path, *, verbose: bool = False) -> None:
    """"""
    directory = str(path)
    if verbose:
        print(f"Opening the directory ({directory}) using the systems default file manager")

    if sys.platform == "win32":
        subprocess.Popen(["start", directory], shell=True)

    elif sys.platform == "darwin":
        subprocess.Popen(["open", directory])

    else:
        # try:
        subprocess.Popen(["xdg-open", directory])
        # except OSError:


def open_app_path(
    app_path: AppPath,
    sub_dir: AppPathSubDirEnum,
    site: bool = False,
    verbose: bool = False,
) -> None:
    """"""
    if not site:
        if sub_dir == AppPathSubDirEnum.data:
            directory = app_path.user_data
        elif sub_dir == AppPathSubDirEnum.config:
            directory = app_path.user_config
        elif sub_dir == AppPathSubDirEnum.cache:
            directory = app_path.user_cache
        elif sub_dir == AppPathSubDirEnum.log:
            directory = app_path.user_log
        else:
            raise NotADirectoryError(f"{sub_dir} not in user options (data,config,cache,logs)")
    else:
        if sub_dir == AppPathSubDirEnum.data:
            directory = app_path.site_data
        elif sub_dir == AppPathSubDirEnum.config:
            directory = app_path.site_config
        else:
            raise NotADirectoryError(f"{sub_dir} not in site options (data,config)")

    if verbose:
        print(
            f"Opening the directory ({directory})"
            f" of the {app_path.app_name} app using the default file manager"
        )

    system_open_path(directory, verbose=verbose)


if __name__ == "__main__":

    def aisahd():
        """"""
        from apppath import PROJECT_APP_PATH

        open_app_path(PROJECT_APP_PATH, AppPathSubDirEnum("data"))

    aisahd()
